package com.apache.cxf.upload.download.service;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.List;

import javax.activation.DataHandler;
import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Response.ResponseBuilder;

import org.apache.cxf.jaxrs.ext.multipart.Attachment;
import org.springframework.stereotype.Service;

@Service("fileService")
public class FileServiceImpl implements IFileService {

	public static final String UPLOAD_FILE_SERVER = "D:\\Demo\\upload\\";

	public Response downloadZippedFile() {

		File file = new File("D:/Demo/download/Sample.zip");

		ResponseBuilder responseBuilder = Response.ok((Object) file);
		responseBuilder.header("Content-Disposition", "attachment; filename=\"MyZippedFile.zip\"");
		return responseBuilder.build();
	}

	public Response uploadZippedFile(List<Attachment> attachments, HttpServletRequest request) {

		// local variables
		DataHandler dataHandler = null;
		MultivaluedMap<String, String> multivaluedMap = null;
		String fileName = null;
		InputStream inputStream = null;

		for (Attachment attachment : attachments) {
			dataHandler = attachment.getDataHandler();
			try{
				// get filename to be uploaded
				multivaluedMap = attachment.getHeaders();
				fileName = getFileName(multivaluedMap);

				if(null != fileName && !"".equalsIgnoreCase(fileName)){
					// write & upload file to server
					inputStream = dataHandler.getInputStream();
					writeToFileServer(inputStream, fileName);

					// close the stream
					inputStream.close();
				}
			}
			catch(IOException ioex) {
				ioex.printStackTrace();
			}
			finally {
				// release resources, if any
			}
		}
		return Response.ok("upload success").build();
	}

	/**
	 * 
	 * @param inputStream
	 * @param fileName
	 */
	private void writeToFileServer(InputStream inputStream, String fileName) {

		OutputStream outputStream = null;
		try {
			outputStream = new FileOutputStream(new File(UPLOAD_FILE_SERVER + fileName));
			int read = 0;
			byte[] bytes = new byte[1024];
			while ((read = inputStream.read(bytes)) != -1) {
				outputStream.write(bytes, 0, read);
			}
			outputStream.flush();
			outputStream.close();
		} 
		catch (IOException ioe) {
			ioe.printStackTrace();
		}
		finally{
			//release resource, if any
		}
	}

	/**
	 * 	
	 * @param multivaluedMap
	 * @return
	 */
	private String getFileName(MultivaluedMap<String, String> multivaluedMap) {

		String[] contentDisposition = multivaluedMap.getFirst("Content-Disposition").split(";");
		for (String filename : contentDisposition) {

			if ((filename.trim().startsWith("filename"))) {
				String[] name = filename.split("=");
				String exactFileName = name[1].trim().replaceAll("\"", "");
				return exactFileName;
			}
		}
		return "unknownFile";
	}
}